%{
/*
  This program transforms two DLV style programs to two first-order theories.
  If both are unsatisfiable, the programs are strongly equivalent.
  Copyright (C) 2004 Patrick Traxler

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include"st.h"

extern int yylex();
int yyerror(char *s);

string rel_symb;
int    cnt_arity;
%}

%token SYM VAR
%token OR NOT
%token IMP

%%

program: 
	| rule program
    ;

rule: head '.'          
    | head IMP body '.'
    | IMP body '.'
    ;

head: literal 
    | literal OR head
    ;

body: literal 
    | literal ',' body
    ;

literal: atom 
    | NOT atom 
    ;

atom: SYM { st_RS.add(*yylval, 0); }
    | SYM { rel_symb = *yylval; delete yylval; cnt_arity = 1; } '(' term terml ')' 
    { if (st_RS.exists(rel_symb) && st_RS.getArity(rel_symb) != cnt_arity) {
          cout << "Semantical error: " << rel_symb  << " has false arity." << endl;
          exit(2);
      }      
      st_RS.add(rel_symb, cnt_arity); 
    }        
    ;

terml: 
    | ',' term { cnt_arity++; } terml
    ;

term: SYM { st_CS.add(*yylval); delete yylval; }
    | VAR
    ;

%%

int yyerror(char *s)
{
  cout << "Parser error: " << s << endl;
  exit(2);
}
